/*

Mike DeFehr, September 2009
mike@defehr.com
http://mikedefehr.com
http://winnipeg.sqlpass.org/

*/

USE tempdb
GO

IF OBJECT_ID('ProductStructure') IS NOT NULL DROP TABLE ProductStructure
IF OBJECT_ID('Product') IS NOT NULL DROP TABLE Product

CREATE TABLE Product
(	ProductID			varchar(25) PRIMARY KEY,
	ProductDescription	varchar(255)
)

CREATE TABLE ProductStructure
(	ProductStructureID	int	IDENTITY(1,1) PRIMARY KEY,
	ParentProductID		varchar(25)
		CONSTRAINT FK_ProductStructure_Product_Parent
		FOREIGN KEY REFERENCES Product(ProductID),
	ChildProductID		varchar(25)
		CONSTRAINT FK_ProductStructure_Product_Child
		FOREIGN KEY REFERENCES Product(ProductID),
	QtyPer				decimal(19,4)
)	
GO		

--insert some products

INSERT Product (ProductID, ProductDescription) VALUES
('DRESSER1','Dresser - Style 1'),
('DTOP1',	'Dresser Top - Style 1'),
('DRAWER1',	'Drawer - Style 1'),
('CABDOOR1','Cabinet Door - Style 1'),
('SHELF1',	'Cabinet Shelf - Style 1'),
('DWRBOX',	'Standard Drawer Box'),
('PINE',	'Wood - Pine Species'),
('OAK',		'Wood - Oak Species'),
('NIGHT1',	'Nightstand - Style 1'),
('NTOP1',	'Nightstand TOP - Style 1')


INSERT ProductStructure (ParentProductID,ChildProductID,QtyPer) VALUES
('DRESSER1','DTOP1',1),
('DTOP1','OAK',12),
('DRESSER1','DRAWER1',6),
('DRAWER1','DWRBOX',1),
('DWRBOX','PINE',5),
('DRESSER1','CABDOOR1',2),
('CABDOOR1','OAK',1.5),
('DRESSER1','SHELF1',2),
('SHELF1','OAK',1),
('NIGHT1','NTOP1',1),
('NTOP1','OAK',3),
('NIGHT1','DRAWER1',2)
GO
--build a recursive cte




WITH cte AS
(
	SELECT	ProductID,
			ProductDescription,
			0 AS BOMLevel
	FROM Product
	WHERE ProductID IN ('DRESSER1','NIGHT1')
)
SELECT * FROM cte;















GO
--results to text
WITH cte AS
(
	SELECT	ProductID,
			ProductDescription,
			CAST(1 AS decimal(19,4)) AS QtyPer,
			0 AS BOMLevel,
			CAST('.' + ProductID + '.' as varchar(4000)) AS ProductPath
	FROM Product
	WHERE ProductID IN ('DRESSER1','NIGHT1')
	UNION ALL
	SELECT	pr.ProductID,
			pr.ProductDescription,
			ps.QtyPer,
			cte.BOMLevel + 1,
			CAST(cte.ProductPath + pr.ProductID + '.' AS varchar(4000))
	FROM Product pr
		JOIN ProductStructure ps ON pr.ProductID = ps.ChildProductID
		JOIN cte ON ps.ParentProductID = cte.ProductID
)
SELECT	REPLICATE(' | ',BOMLevel)  +  ProductID
		+ ' ' + ProductDescription
		+ ' (' + CAST(QtyPer AS varchar) + ')'
FROM cte
ORDER BY cte.ProductPath

GO

--Example 2: Use Recursion to do iteration?

IF OBJECT_ID('MonthlySales') IS NOT NULL DROP TABLE MonthlySales
GO
CREATE TABLE MonthlySales
(	YearMonth	date PRIMARY KEY,
	TotalSales	money
)
GO
INSERT MonthlySales (YearMonth,TotalSales) VALUES
('2008-01-01',11568),
('2008-02-01',9685),
('2008-03-01',10458),
('2008-04-01',10245),
('2008-05-01',11365),
('2008-06-01',9548),
('2008-07-01',10493),
('2008-08-01',9126),
('2008-09-01',11276),
('2008-10-01',10785),
('2008-11-01',9456),
('2008-12-01',10739),
('2009-01-01',10759),
('2009-02-01',10862),
('2009-03-01',9459),
('2009-04-01',9948),
('2009-05-01',10057),
('2009-06-01',10457)
GO

--how to get the running sum?


SELECT *, (	SELECT SUM(TotalSales) 
			FROM MonthlySales 
			WHERE YearMonth <= ms.YearMonth
				AND YEAR(YearMonth) = YEAR(ms.YearMonth)	) AS SalesToDate 
FROM MonthlySales ms;


WITH cte AS
(
	SELECT	YearMonth,
			TotalSales,
			TotalSales AS YTDSales
	FROM MonthlySales
	WHERE YearMonth = '2008-01-01'
	UNION ALL
	SELECT	ms.YearMonth,
			ms.TotalSales,
			CASE WHEN YEAR(ms.YearMonth) <> YEAR(cte.YearMonth) 
				THEN ms.TotalSales
				ELSE cte.YTDSales + ms.TotalSales
			END
	FROM cte
		JOIN MonthlySales ms ON DATEADD(month,1,cte.YearMonth) = ms.YearMonth
)
SELECT * FROM cte
OPTION (MAXRECURSION 0)



